/*************************************************************************
Title:    Testing output to a HD44780 based LCD display.
Author:   Peter Fleury  <pfleury@gmx.ch>  http://tinyurl.com/peterfleury
File:     $Id: test_lcd.c,v 1.8 2015/01/31 18:04:08 peter Exp $
Software: AVR-GCC 4.x
Hardware: HD44780 compatible LCD text display
          AVR with external SRAM interface if memory-mapped LCD interface is used
          any AVR with 7 free I/O pins if 4-bit IO port mode is used
**************************************************************************/

#define F_CPU 4000000UL

#include <stdlib.h>
#include <stdio.h>
#include <avr/io.h>
#include <avr/eeprom.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "lcd.h"


// Game menu
const char game_1[] PROGMEM = "AstraSuperStars ";
const char game_2[] PROGMEM = "BakuBakuAnimal  ";
const char game_3[] PROGMEM = "Batman Forever  ";
const char game_4[] PROGMEM = "Columns 97      ";
const char game_5[] PROGMEM = "Cotton 2        ";
const char game_6[] PROGMEM = "Cotton Boomerang";
const char game_7[] PROGMEM = "DanchideHanafuda";
const char game_8[] PROGMEM = "Decathlete      ";
const char game_9[] PROGMEM = "Die Hard Arcade ";
const char game_10[] PROGMEM = "Touryuu Densetsu";
const char game_11[] PROGMEM = "FinalFightRvenge";
const char game_12[] PROGMEM = "FunkyHeadBoxers ";
const char game_13[] PROGMEM = "Find Love       ";
const char game_14[] PROGMEM = "Final Arch      ";
const char game_15[] PROGMEM = "GoldenAxeTheDuel";
const char game_16[] PROGMEM = "Guardian Force  ";
const char game_17[] PROGMEM = "Groove On Fight ";
const char game_18[] PROGMEM = "Hana.Taisn.Clmns";
const char game_19[] PROGMEM = "Intro Don Don   ";
const char game_20[] PROGMEM = "Pro Mahjong KwmS";
const char game_21[] PROGMEM = "Maru-Chan de Goo";
const char game_22[] PROGMEM = "Mausuke no Ojama";
const char game_23[] PROGMEM = "MyFairLady:Mahj ";
const char game_24[] PROGMEM = "OthelloShiryouyo";
const char game_25[] PROGMEM = "PebbleBeachGolf ";
const char game_26[] PROGMEM = "PrincessKurara  ";
const char game_27[] PROGMEM = "Puyo Puyo Sun   ";
const char game_28[] PROGMEM = "RadiantSilvergun";
const char game_29[] PROGMEM = "Sando-R:Puzzle  ";
const char game_30[] PROGMEM = "Sea Bass Fishing";
const char game_31[] PROGMEM = "Shangai:TheGrtWl";
const char game_32[] PROGMEM = "Shienryu        ";
const char game_33[] PROGMEM = "Soukyugurentai  ";
const char game_34[] PROGMEM = "SteepSlopeSlders";
const char game_35[] PROGMEM = "Suikoenbu       ";
const char game_36[] PROGMEM = "TecmoWorldCup98 ";
const char game_37[] PROGMEM = "Vrt.Fighter Kids";
const char game_38[] PROGMEM = "Vrt.FighterRemix";
const char game_39[] PROGMEM = "Winter Heat     ";
const char game_40[] PROGMEM = "ProWrestFeatVrtu";
const char game_41[] PROGMEM = "EjihonTanJimusyo";
const char game_42[] PROGMEM = "DaeJeon!SanJeSJe";
const char game_43[] PROGMEM = "Critter Crusher ";
const char game_44[] PROGMEM = "MagicalZunouPowr";
const char game_45[] PROGMEM = "Mic.BattleCharge";
const char game_46[] PROGMEM = "Name Clube      ";
const char game_47[] PROGMEM = "Print Club 2    ";
const char game_48[] PROGMEM = "Print Club 2 v3 ";
const char game_49[] PROGMEM = "Print Club Pokem";
const char game_50[] PROGMEM = "Stress Buster   ";
const char game_51[] PROGMEM = "TechnicalBowling";
const char game_52[] PROGMEM = "Virtual Mahjong ";
const char game_53[] PROGMEM = "ChoroQ          ";
const char game_54[] PROGMEM = "EMPTY SLOT 1    ";
const char game_55[] PROGMEM = "EMPTY SLOT 2    ";
const char game_56[] PROGMEM = "EMPTY SLOT 3    ";
const char game_57[] PROGMEM = "EMPTY SLOT 4    ";
const char game_58[] PROGMEM = "EMPTY SLOT 5    ";
const char game_59[] PROGMEM = "EMPTY SLOT 6    ";
const char game_60[] PROGMEM = "Dynamite Deka   ";
const char game_61[] PROGMEM = "SuperMajorLeague";

PGM_P const game_list[61] PROGMEM =
{
	game_1,game_2,game_3,game_4,game_5,game_6,game_7,game_8,game_9,game_10,
	game_11,game_12,game_13,game_14,game_15,game_16,game_17,game_18,game_19,game_20,
	game_21,game_22,game_23,game_24,game_25,game_26,game_27,game_28,game_29,game_30,
	game_31,game_32,game_33,game_34,game_35,game_36,game_37,game_38,game_39,game_40,
	game_41,game_42,game_43,game_44,game_45,game_46,game_47,game_48,game_49,game_50,
	game_51,game_52,game_53,game_54,game_55,game_56,game_57,game_58,game_59,game_60,
	game_61
};

#define bit_is_clear(sfr,bit) (!(_SFR_BYTE(sfr) & _BV(bit)))

unsigned int game_index = 0;

void display_game_name(unsigned int index)
{
	char buffer[16];
	PGM_P p;
	
	// Display game number
	lcd_gotoxy(0,0);
	sprintf(buffer,"GAME %02d",index+1);
	lcd_puts(buffer);
	
	// Display game name
    lcd_gotoxy(0,1);
    memcpy_P(&p, &game_list[index], sizeof(PGM_P));
    strcpy_P(buffer,p);
    lcd_puts(buffer);
}

void select_game(unsigned int index)
{
	// Set DIP Switches
	// Important!!! Do NOT send HIGH signal, just change the DDRC pins to HiZ for logic 1
	// This is the safe way to interface 3.3V logic of the multi.
	DDRC = 0x00;
	PORTC = ~(index+1);
	DDRC = index+1;	
	
	eeprom_write_byte(0,index);
	while(bit_is_clear(PIND,PD2)) _delay_ms(1);
	_delay_ms(500);
	
	//Reset the multi (Just change the DDRD from HiZ to LOW)
	DDRD = (1 << PD3);
	_delay_ms(100);
	DDRD = (0 << PD3);
}

int main(void)
{

	// Read last selected game number from EEPROM
	game_index = eeprom_read_byte(0);
	if (game_index > 60) game_index = 0;
	
	// Initialize ports
	DDRD = 0x00;
	PORTD = 0b11110111; // PD3 is set as output. controls the reset line to multi

	DDRC = 0x00;
	PORTC= 0xff;

	// Initialize LCD
    lcd_init(LCD_DISP_ON);
	display_game_name(game_index);

	// Display last selected game (if any)
	if (game_index > 0) select_game(game_index);

    while(1)                     
    {

		// Check the buttons
		if (bit_is_clear(PIND,PD0))			/* Up */
		{ 
			if (game_index > 0) game_index--;

			display_game_name(game_index);

			while(bit_is_clear(PIND,PD0)) _delay_ms(1);

		} 
		
		if (bit_is_clear(PIND,PD1))	/* Down */
		{
			if (game_index < 60) game_index++;

			display_game_name(game_index);

			while(bit_is_clear(PIND,PD1)) _delay_ms(1);
		}
		
		if (bit_is_clear(PIND,PD2))	/* Select */
		{
			select_game(game_index);
		}
		_delay_ms(50);    
    }
}

